
#  BankPolymorphic.py (c) Kari Laitinen

#  http://www.naturalprogramming.com

#  2006-04-27  File created.
#  2022-12-18  Converted to Python 3.

class  BankAccount :

   def __init__( self, given_account_owner,
                       given_account_number,
                       initial_balance ) :

      self.account_owner    =  given_account_owner
      self.account_number   =  given_account_number
      self.account_balance  =  initial_balance


   def show_account_data( self ) :

      print( "\n\nB A N K   A C C O U N T   D A T A : " )
      print( "   Account owner :  %s"  %  self.account_owner )
      print( "   Account number:  %d"  %  self.account_number )
      print( "   Current balance: %.2f"  %  self.account_balance )


   def deposit_money( self, amount_to_deposit ) :

      print( "\n\nTRANSACTION FOR ACCOUNT OF %s (Account number %d)"  %  \
            ( self.account_owner, self.account_number ), end="" )
      print( "\n   Amount deposited: %.2f\n   Old account balance: %.2f"  %  \
            ( amount_to_deposit, self.account_balance ), end="" )
      self.account_balance  =  self.account_balance  +  amount_to_deposit
      print( "   New balance: %.2f"  %  self.account_balance, end="" )


   def withdraw_money( self, amount_to_withdraw ) :
   
      print( "\n\nTRANSACTION FOR ACCOUNT OF %s (Account number %d)"  %  \
            ( self.account_owner, self.account_number ), end="" )

      if  self.account_balance  <  amount_to_withdraw :

         print( "\n   -- Transaction not completed: Not enough money"  \
               " to withdraw %.2f"  %  amount_to_withdraw, end="" )

      else :

         print( "\n   Amount withdrawn:    %.2f\n   Old account balance: %.2f"  % \
               ( amount_to_withdraw, self.account_balance ), end="" )
         self.account_balance  =  self.account_balance  -  amount_to_withdraw
         print( "   New balance: %.2f"  %  self.account_balance, end="" )
   

   def transfer_money_to( self, receiving_account,
                                amount_to_transfer ) :

      print( "\n\nTRANSACTION FOR ACCOUNT OF %s (Account number %d)"  %  \
            ( self.account_owner, self.account_number ), end="" )

      if  self.account_balance  >=  amount_to_transfer :

         receiving_account.account_balance  =  \
            receiving_account.account_balance  +  amount_to_transfer

         print( "\n   %.2f was transferred to %s (Account no. %d)."  %   \
               ( amount_to_transfer, receiving_account.account_owner,
                 receiving_account.account_number ), end="" )
         print( "\n   Balance before transfer: %.2f"  %  self.account_balance, end="" )
         self.account_balance  =  self.account_balance  -  amount_to_transfer,
         print( "   New balance: %.2f"  %  self.account_balance, end="" )
   
      else :
      
         print( "\n   -- Not enough money for transfer.", end="" )


class  AccountWithCredit ( BankAccount ) :

   def __init__( self, given_account_owner,
                       given_account_number,
                       initial_balance,
                       given_credit_limit  ) :
 
      BankAccount.__init__( self, given_account_owner,
                                  given_account_number,
                                  initial_balance )

      self.credit_limit  =  given_credit_limit


   def withdraw_money( self, amount_to_withdraw ) :
   
      print( "\n\nTRANSACTION FOR ACCOUNT OF %s (Account number %d)"  %  \
            ( self.account_owner, self.account_number ), end="" )

      if  self.account_balance  +  self.credit_limit  <  amount_to_withdraw  :

         print( "\n   --  Transaction not completed: "  \
               "Not enough credit to withdraw %.2f"  %  amount_to_withdraw, end="" )

      else :
      
         print( "\n   Amount withdrawn:    %.2f\n   Old account balance: %.2f"  % \
               ( amount_to_withdraw, self.account_balance ), end="" )
         self.account_balance  =  self.account_balance  -  amount_to_withdraw
         print( "   New balance: %.2f"  %  self.account_balance, end="" )
   



class  RestrictedAccount  ( BankAccount ) :

   def __init__( self, given_account_owner,
                       given_account_number,
                       initial_balance,
                       given_withdrawal_limit  ) :

      BankAccount.__init__( self, given_account_owner,
                                  given_account_number,
                                  initial_balance )

      self.maximum_amount_to_withdraw  =  given_withdrawal_limit


   def withdraw_money( self, amount_to_withdraw ) :

      if  amount_to_withdraw  >  self.maximum_amount_to_withdraw  :
      
         print( "\n\nTRANSACTION FOR ACCOUNT OF %s (Account number %d)"  %  \
               ( self.account_owner, self.account_number ), end="" )

         print( "\n   -- Transaction not completed: Cannot withdraw %.2f"  \
               "\n   -- Withdrawal limit is %.2f."  %  ( amount_to_withdraw,
               self.maximum_amount_to_withdraw ), end="" )
   
      else :

         BankAccount.withdraw_money( self, amount_to_withdraw )



#  The main program begins.

beatles_account  =  BankAccount( "John Lennon", 222222, 2000.00 ) 

stones_account   =  AccountWithCredit( "Brian Jones", 333333,
                                        2000.00,  1000.00 )

doors_account    =  RestrictedAccount( "Jim Morrison", 444444,
                                        4000.00,  1000.00 )

beatles_account.withdraw_money( 2500.00 )
stones_account.withdraw_money( 2500.00 )
doors_account.withdraw_money( 2500.00 )




